//****************************************************************************************
//
//		Copyright(c) 2004 LeerSoft, L.L.C.
//		All rights reserved. The software and associated documentation supplied hereunder
//		are the proprietary information of LeerSoft, L.L.C. However, feel free to use the
//		the software routines contained in these works in your commercial and/or
//		noncommercial software products provided:
//
//		1. Proper copyright notice in the form "Copyright(C) 2004 LeerSoft, L.L.C." is
//			shown to the user anywhere your own copyright may appear.
//		2. That this work, substantially unmodified, is NOT offered for sale by you
//			as a stand-alone work.
//		3. This copyright may not be removed from this file.
//
//		http://www.leersoft.com
//
//****************************************************************************************

using System;
using System.ComponentModel.Design;
using System.Drawing;
using System.Drawing.Imaging;
using System.Collections;
using System.ComponentModel;
using System.IO;
using System.Reflection;
using System.Windows.Forms;


namespace LeerSoft.Tools
{
	/// <summary>
	/// The LeerColorPicker is a compact and powerful color selection dialog. If offers many special features:
	/// <para>. A color selection grid ordered by hue, saturation and brightness.
	/// (see http://www.nebulus.org/tutorials/2d/photoshop/color/color2.html for a discussion of these terms.) This
	/// grid makes it easier to select various colors using the crosshairs than the .Net ColorDialog. You can
	/// then adjust the brightness by varying the brightness selector bar.</para>
	/// <para>. Color wells for specific color selection can display basic colors or "web-safe" colors. These
	/// wells can be sorted in various ways selectable by the user or by the caller. The sorting selector can
	/// be turned of by the AllowColorSort property.</para>
	/// <para>. When the user selects a color via the crosshairs and brightness bar, he/she can then press the
	/// Convert To Web (or Convert To Basic if only basic colors are displayed) to have the LeerColorPicker
	/// choose the color that is the closest match within the available Web (or basic) colors.</para>
	/// <para>. Users can select colors by the old standby RGB values by entering the appropriate values in the
	/// R, G and B text boxes.</para>
	/// <para>. Custom colors add/remove is well defined and easy to use.</para>	
	/// <para>. The hex value of the color as would be used in an HTML page is always shown in the color bar
	/// which shows a sample of the selected color.</para>
	/// </summary>
	[ToolboxBitmapAttribute(typeof(LeerColorPicker), "Dialogs.LeerColorPicker.ico")]
	[ToolboxItem(true)]
	[DesignTimeVisible(true)]
	[Designer(typeof(LeerColorPicker.LeerColorDialogDesigner))]
	[DefaultEvent("Click")]
	public class LeerColorPicker : System.Windows.Forms.Form
	{
		private bool fAllowSort = true;	// true if user can change sort order of colors
		private bool fAllowWebColors = true;	// true if web colors are allowed
		private bool fColorsHasFocus = false; // pictColors has focus flag
		private bool fPointerHasFocus = false;// pointer next to shades box has focus
		private bool fMouseDown = false; // needed for mouse move events
		private bool fPtrMouseDown = false; // mouse down in shade bar ptr
		private bool fUseBasicColors = true;	// use basic colors if true
		private char[] hexDigits = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9','A', 'B','C', 'D', 'E', 'F'};
		private Color[] arrayBasicColors = new Color[40];		// allow for 40 basic colors
		/// <summary>
		/// Array of 16 colors that appear in the custom color picker wells. "White" is considered an empty slot for new colors.
		/// </summary>
		public Color[] CustomColors = {Color.White, Color.White, Color.White, Color.White, Color.White, Color.White, Color.White, 
			Color.White, Color.White, Color.White, Color.White, Color.White, Color.White, Color.White, Color.White, Color.White};
		private Color[] arrayWebColors = new Color[140];		// allow for 140 web colors
		private Color[] curSwatchArray = null;						// points to swatches currently being used
		private Color colorButton = Color.FromKnownColor(KnownColor.Control); // button color
		private Color colorSelected = Color.SkyBlue; //initial color (somewhere in mid-HSB)
		private ColorSortOrder colorSortOrder = ColorSortOrder.Hue; // sort order to use
		private ColorSortOrder prevColorSortOrder = ColorSortOrder.Unknown;	// sort order last time thru SortColors()
		private LeerSkin skin = null;	// skin look and feel
		private int customWellFocused = -1; // any custom well have focus?
		private int customWellSelected = -1; // current selected custom well
		private int lastYforPtr = 0;	// save for last "y" coordinate where ptr painted
		private int xPos = 120;		// cur x position in image (range 1-240)
		private int yPos = 100;		// cur y position in image (range 1-200)
		private int vPos = 100;		// "value" position in HSV model (range 1-200)
		private int rValue = 128;		// rgb for xPos, yPos, vPos
		private int gValue = 128;
		private int bValue = 128;
		private int wellCols = 8;		// number of rows for color wells
		private int wellHeight = 30;	// well cell height
		private int wellRows = 5;		// number of rows of wells
		private int wellSelected = -1; // current selected well
		private int wellFocused = -1;	// current focused well
		private int wellSpace = 3;		// space around well
		private int wellSwatchWidth = 21; // dim of swatch square
		private int wellWidth = 27;	// width of well
		private String ERR_CUSTOMCOLOREXISTS = "Current color already in custom colors.\n\nDo you want to add it again?";
		private String ERR_NOCUSTOMSLOTSOPEN = "No available custom slots. Please select\nand then remove some custom color.";
		private String TIT_CUSTOMCOLOREXISTS = "Color Already Custom";
		private String TIT_NOCUSTOMSLOTSOPEN = "Cannot Add Custom Color";
		private System.Drawing.Bitmap imageBasicColors = new System.Drawing.Bitmap(226, 161);
		private System.Drawing.Bitmap imageColors = new System.Drawing.Bitmap(240, 200);	// colors box
		private System.Drawing.Bitmap imageCustomColors = new System.Drawing.Bitmap(256, 40);
		private System.Drawing.Bitmap imageShades = new System.Drawing.Bitmap(16, 200);
		private System.Drawing.Bitmap imageWebColors = new System.Drawing.Bitmap(226, 161);
		private System.Drawing.Bitmap imageShadePtr = new System.Drawing.Bitmap(9, 217);	// shades bar ptr
		
		// Color arrays for basic colors and web colors
		
		private Color[] colorsBasic = {	Color.Black, Color.Brown, Color.DarkOliveGreen, Color.DarkGreen,
													Color.Teal, Color.DarkBlue, Color.Indigo, Color.DimGray,
													Color.Maroon, Color.SaddleBrown, Color.Olive, Color.Green,
													Color.Purple, Color.Blue, Color.SteelBlue, Color.Gray,
													Color.Red, Color.Orange, Color.Lime, Color.SeaGreen,
													Color.Aqua, Color.LightBlue, Color.Violet, Color.DarkGray,
													Color.Pink, Color.Gold, Color.Yellow, Color.LawnGreen,
													Color.Turquoise, Color.DeepSkyBlue, Color.Plum, Color.LightGray,
													Color.Navy, Color.Tan, Color.PaleGoldenrod, Color.PaleGreen,
													Color.PaleTurquoise, Color.SkyBlue, Color.Thistle, Color.White };

		// VS.Net automatic declarations
		private LeerSoft.Tools.LeerPictureBox pictColors;
		private System.Windows.Forms.Panel panelCustom;
		private System.Windows.Forms.PictureBox pictColorSample;
		private LeerSoft.Tools.LeerPictureBox pictShades;
		private System.Windows.Forms.Label labelRed;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.TextBox textBoxRed;
		private System.Windows.Forms.TextBox textBoxGreen;
		private System.Windows.Forms.TextBox textBoxBlue;
		private System.Windows.Forms.RadioButton radioBtnBasic;
		private System.Windows.Forms.RadioButton radioButton2;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.GroupBox groupColors;
		private LeerSoft.Tools.LeerPictureBox pictColorSwatches;
		private System.Windows.Forms.GroupBox groupCustom;
		private System.Windows.Forms.Label label4;
		private System.Windows.Forms.Panel panelRadioButtons;
		private System.Windows.Forms.ToolTip toolTipSwatches;
		private LeerSoft.Tools.LeerButton leerBtnConvertColor;
		private LeerSoft.Tools.LeerButton btnOK;
		private LeerSoft.Tools.LeerButton btnCancel;
		private LeerSoft.Tools.LeerPictureBox pictCustomSwatches;
		private LeerSoft.Tools.LeerButton btnAddCustom;
		private LeerSoft.Tools.LeerButton btnRemoveCustom;
		private LeerSoft.Tools.LeerPictureBox pictPointer;
		private System.Windows.Forms.Panel panelSwatches;
		private System.Windows.Forms.Panel panelCustomSwatches;
		private System.Windows.Forms.Label label5;
		private System.Windows.Forms.ComboBox cbxSortOrder;
		private System.ComponentModel.IContainer components;

		/// <summary>
		/// Color picker dialog
		/// </summary>
		public LeerColorPicker()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
			Array knownColors = Enum.GetValues( typeof(System.Drawing.KnownColor) );

			int nColors = 0;

			foreach(KnownColor k in knownColors)
			{
				Color c = Color.FromKnownColor(k);
				if(!c.IsSystemColor && (c.A > 0))
				{
					arrayWebColors[nColors++] = c;
					if(nColors >= 140)				// just in case we get too many
						break;
				}
			}
			Array.Copy(colorsBasic, arrayBasicColors, colorsBasic.Length);	// get basic colors to array
		}

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="container"></param>
		public LeerColorPicker(IContainer container): this()
		{
			container.Add(this);
		}
		

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(LeerColorPicker));
			this.pictColors = new LeerSoft.Tools.LeerPictureBox();
			this.pictShades = new LeerSoft.Tools.LeerPictureBox();
			this.panelCustom = new System.Windows.Forms.Panel();
			this.groupCustom = new System.Windows.Forms.GroupBox();
			this.label5 = new System.Windows.Forms.Label();
			this.panelCustomSwatches = new System.Windows.Forms.Panel();
			this.pictCustomSwatches = new LeerSoft.Tools.LeerPictureBox();
			this.pictPointer = new LeerSoft.Tools.LeerPictureBox();
			this.btnRemoveCustom = new LeerSoft.Tools.LeerButton();
			this.btnAddCustom = new LeerSoft.Tools.LeerButton();
			this.textBoxBlue = new System.Windows.Forms.TextBox();
			this.textBoxGreen = new System.Windows.Forms.TextBox();
			this.textBoxRed = new System.Windows.Forms.TextBox();
			this.label3 = new System.Windows.Forms.Label();
			this.label2 = new System.Windows.Forms.Label();
			this.labelRed = new System.Windows.Forms.Label();
			this.pictColorSample = new System.Windows.Forms.PictureBox();
			this.groupColors = new System.Windows.Forms.GroupBox();
			this.panelSwatches = new System.Windows.Forms.Panel();
			this.pictColorSwatches = new LeerSoft.Tools.LeerPictureBox();
			this.radioBtnBasic = new System.Windows.Forms.RadioButton();
			this.radioButton2 = new System.Windows.Forms.RadioButton();
			this.label1 = new System.Windows.Forms.Label();
			this.label4 = new System.Windows.Forms.Label();
			this.panelRadioButtons = new System.Windows.Forms.Panel();
			this.toolTipSwatches = new System.Windows.Forms.ToolTip(this.components);
			this.leerBtnConvertColor = new LeerSoft.Tools.LeerButton();
			this.btnOK = new LeerSoft.Tools.LeerButton();
			this.btnCancel = new LeerSoft.Tools.LeerButton();
			this.cbxSortOrder = new System.Windows.Forms.ComboBox();
			this.panelCustom.SuspendLayout();
			this.groupCustom.SuspendLayout();
			this.panelCustomSwatches.SuspendLayout();
			this.groupColors.SuspendLayout();
			this.panelSwatches.SuspendLayout();
			this.panelRadioButtons.SuspendLayout();
			this.SuspendLayout();
			// 
			// pictColors
			// 
			this.pictColors.CaptureArrowKeys = true;
			this.pictColors.Image = null;
			this.pictColors.Location = new System.Drawing.Point(8, 20);
			this.pictColors.Name = "pictColors";
			this.pictColors.Size = new System.Drawing.Size(240, 200);
			this.pictColors.TabIndex = 4;
			this.pictColors.Enter += new System.EventHandler(this.pictColorsFocusChange);
			this.pictColors.MouseUp += new System.Windows.Forms.MouseEventHandler(this.pictColorsMouseUp);
			this.pictColors.Paint += new System.Windows.Forms.PaintEventHandler(this.pictColorsPaint);
			this.pictColors.KeyDown += new System.Windows.Forms.KeyEventHandler(this.pictColorsKeyDown);
			this.pictColors.MouseMove += new System.Windows.Forms.MouseEventHandler(this.pictColorsMouseMove);
			this.pictColors.MouseDown += new System.Windows.Forms.MouseEventHandler(this.pictColorsMouseDown);
			this.pictColors.Leave += new System.EventHandler(this.pictColorsLeave);
			// 
			// pictShades
			// 
			this.pictShades.CaptureArrowKeys = true;
			this.pictShades.Image = null;
			this.pictShades.Location = new System.Drawing.Point(252, 20);
			this.pictShades.Name = "pictShades";
			this.pictShades.Size = new System.Drawing.Size(16, 200);
			this.pictShades.TabIndex = 6;
			this.pictShades.Enter += new System.EventHandler(this.pictShadesEnter);
			this.pictShades.KeyDown += new System.Windows.Forms.KeyEventHandler(this.pictShadesKeyDown);
			this.pictShades.MouseDown += new System.Windows.Forms.MouseEventHandler(this.pictShadesMouseDown);
			this.pictShades.Leave += new System.EventHandler(this.pictShadesLeave);
			// 
			// panelCustom
			// 
			this.panelCustom.Controls.Add(this.groupCustom);
			this.panelCustom.Dock = System.Windows.Forms.DockStyle.Right;
			this.panelCustom.Location = new System.Drawing.Point(244, 0);
			this.panelCustom.Name = "panelCustom";
			this.panelCustom.Size = new System.Drawing.Size(284, 338);
			this.panelCustom.TabIndex = 7;
			// 
			// groupCustom
			// 
			this.groupCustom.Controls.Add(this.label5);
			this.groupCustom.Controls.Add(this.panelCustomSwatches);
			this.groupCustom.Controls.Add(this.pictPointer);
			this.groupCustom.Controls.Add(this.btnRemoveCustom);
			this.groupCustom.Controls.Add(this.btnAddCustom);
			this.groupCustom.Controls.Add(this.textBoxBlue);
			this.groupCustom.Controls.Add(this.pictShades);
			this.groupCustom.Controls.Add(this.textBoxGreen);
			this.groupCustom.Controls.Add(this.pictColors);
			this.groupCustom.Controls.Add(this.textBoxRed);
			this.groupCustom.Controls.Add(this.label3);
			this.groupCustom.Controls.Add(this.label2);
			this.groupCustom.Controls.Add(this.labelRed);
			this.groupCustom.Dock = System.Windows.Forms.DockStyle.Fill;
			this.groupCustom.Location = new System.Drawing.Point(0, 0);
			this.groupCustom.Name = "groupCustom";
			this.groupCustom.Size = new System.Drawing.Size(284, 338);
			this.groupCustom.TabIndex = 0;
			this.groupCustom.TabStop = false;
			this.groupCustom.Text = "Custom Color Picker";
			// 
			// label5
			// 
			this.label5.Location = new System.Drawing.Point(8, 260);
			this.label5.Name = "label5";
			this.label5.Size = new System.Drawing.Size(56, 36);
			this.label5.TabIndex = 24;
			this.label5.Text = "Custom Colors:";
			// 
			// panelCustomSwatches
			// 
			this.panelCustomSwatches.Controls.Add(this.pictCustomSwatches);
			this.panelCustomSwatches.Location = new System.Drawing.Point(71, 259);
			this.panelCustomSwatches.Name = "panelCustomSwatches";
			this.panelCustomSwatches.Size = new System.Drawing.Size(202, 42);
			this.panelCustomSwatches.TabIndex = 23;
			// 
			// pictCustomSwatches
			// 
			this.pictCustomSwatches.CaptureArrowKeys = true;
			this.pictCustomSwatches.Image = null;
			this.pictCustomSwatches.Location = new System.Drawing.Point(0, 0);
			this.pictCustomSwatches.Name = "pictCustomSwatches";
			this.pictCustomSwatches.Size = new System.Drawing.Size(200, 40);
			this.pictCustomSwatches.TabIndex = 18;
			this.pictCustomSwatches.Enter += new System.EventHandler(this.pickCustomSwatchesEnter);
			this.pictCustomSwatches.MouseUp += new System.Windows.Forms.MouseEventHandler(this.pictCustomSwatchesMouseUp);
			this.pictCustomSwatches.KeyDown += new System.Windows.Forms.KeyEventHandler(this.pictCustomSwatchesKeyDown);
			this.pictCustomSwatches.MouseMove += new System.Windows.Forms.MouseEventHandler(this.pictCustomSwatchesMouseMove);
			this.pictCustomSwatches.MouseLeave += new System.EventHandler(this.pictCustomSwatchesMouseLeave);
			this.pictCustomSwatches.Leave += new System.EventHandler(this.pictCustomSwatchesLeave);
			// 
			// pictPointer
			// 
			this.pictPointer.CaptureArrowKeys = true;
			this.pictPointer.Image = null;
			this.pictPointer.Location = new System.Drawing.Point(268, 12);
			this.pictPointer.Name = "pictPointer";
			this.pictPointer.Size = new System.Drawing.Size(9, 216);
			this.pictPointer.TabIndex = 22;
			this.pictPointer.Enter += new System.EventHandler(this.pictPointerEnter);
			this.pictPointer.MouseUp += new System.Windows.Forms.MouseEventHandler(this.pictPointerMouseUp);
			this.pictPointer.KeyDown += new System.Windows.Forms.KeyEventHandler(this.pictShadesKeyDown);
			this.pictPointer.MouseMove += new System.Windows.Forms.MouseEventHandler(this.pictPointerMouseMove);
			this.pictPointer.MouseDown += new System.Windows.Forms.MouseEventHandler(this.pictPointerMouseDown);
			this.pictPointer.Leave += new System.EventHandler(this.pictPointerLeave);
			// 
			// btnRemoveCustom
			// 
			this.btnRemoveCustom.AccessibleRole = System.Windows.Forms.AccessibleRole.PushButton;
			this.btnRemoveCustom.BorderColor = System.Drawing.Color.Black;
			this.btnRemoveCustom.DisabledColor = System.Drawing.Color.FromArgb(((System.Byte)(180)), ((System.Byte)(177)), ((System.Byte)(163)));
			this.btnRemoveCustom.DropArrowImage = ((System.Drawing.Image)(resources.GetObject("btnRemoveCustom.DropArrowImage")));
			this.btnRemoveCustom.FlatStyle = LeerSoft.Tools.LeerFlatStyle.ThreeD;
			this.btnRemoveCustom.Font = new System.Drawing.Font("Microsoft Sans Serif", 7.8F);
			this.btnRemoveCustom.HighlightColor = System.Drawing.Color.LightGray;
			this.btnRemoveCustom.ImageAlign = System.Drawing.ContentAlignment.MiddleCenter;
			this.btnRemoveCustom.Location = new System.Drawing.Point(132, 308);
			this.btnRemoveCustom.Name = "btnRemoveCustom";
			this.btnRemoveCustom.Size = new System.Drawing.Size(148, 24);
			this.btnRemoveCustom.Style = LeerSoft.Tools.LeerButtonStyle.PushButton;
			this.btnRemoveCustom.TabIndex = 21;
			this.btnRemoveCustom.Text = "Remove Custom Color";
			this.btnRemoveCustom.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			this.btnRemoveCustom.TextPadding.Bottom = 0;
			this.btnRemoveCustom.TextPadding.Left = 0;
			this.btnRemoveCustom.TextPadding.Right = 0;
			this.btnRemoveCustom.TextPadding.Top = 0;
			this.btnRemoveCustom.Click += new System.EventHandler(this.btnRemoveCustomClick);
			// 
			// btnAddCustom
			// 
			this.btnAddCustom.AccessibleRole = System.Windows.Forms.AccessibleRole.PushButton;
			this.btnAddCustom.BorderColor = System.Drawing.Color.Black;
			this.btnAddCustom.DisabledColor = System.Drawing.Color.FromArgb(((System.Byte)(180)), ((System.Byte)(177)), ((System.Byte)(163)));
			this.btnAddCustom.DropArrowImage = ((System.Drawing.Image)(resources.GetObject("btnAddCustom.DropArrowImage")));
			this.btnAddCustom.FlatStyle = LeerSoft.Tools.LeerFlatStyle.ThreeD;
			this.btnAddCustom.Font = new System.Drawing.Font("Microsoft Sans Serif", 7.8F);
			this.btnAddCustom.HighlightColor = System.Drawing.Color.LightGray;
			this.btnAddCustom.ImageAlign = System.Drawing.ContentAlignment.MiddleCenter;
			this.btnAddCustom.Location = new System.Drawing.Point(4, 308);
			this.btnAddCustom.Name = "btnAddCustom";
			this.btnAddCustom.Size = new System.Drawing.Size(124, 24);
			this.btnAddCustom.Style = LeerSoft.Tools.LeerButtonStyle.PushButton;
			this.btnAddCustom.TabIndex = 19;
			this.btnAddCustom.Text = "Add Custom Color";
			this.btnAddCustom.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			this.btnAddCustom.TextPadding.Bottom = 0;
			this.btnAddCustom.TextPadding.Left = 0;
			this.btnAddCustom.TextPadding.Right = 0;
			this.btnAddCustom.TextPadding.Top = 0;
			this.btnAddCustom.Click += new System.EventHandler(this.btnAddCustomClick);
			// 
			// textBoxBlue
			// 
			this.textBoxBlue.Location = new System.Drawing.Point(236, 232);
			this.textBoxBlue.Name = "textBoxBlue";
			this.textBoxBlue.Size = new System.Drawing.Size(40, 22);
			this.textBoxBlue.TabIndex = 17;
			this.textBoxBlue.Text = "";
			this.textBoxBlue.Validating += new System.ComponentModel.CancelEventHandler(this.textBoxBlueValidate);
			// 
			// textBoxGreen
			// 
			this.textBoxGreen.Location = new System.Drawing.Point(144, 232);
			this.textBoxGreen.Name = "textBoxGreen";
			this.textBoxGreen.Size = new System.Drawing.Size(40, 22);
			this.textBoxGreen.TabIndex = 16;
			this.textBoxGreen.Text = "";
			this.textBoxGreen.Validating += new System.ComponentModel.CancelEventHandler(this.textBoxGreenValidate);
			// 
			// textBoxRed
			// 
			this.textBoxRed.Location = new System.Drawing.Point(44, 232);
			this.textBoxRed.Name = "textBoxRed";
			this.textBoxRed.Size = new System.Drawing.Size(40, 22);
			this.textBoxRed.TabIndex = 15;
			this.textBoxRed.Text = "";
			this.textBoxRed.Validating += new System.ComponentModel.CancelEventHandler(this.textBoxRedValidating);
			// 
			// label3
			// 
			this.label3.Location = new System.Drawing.Point(192, 232);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(38, 23);
			this.label3.TabIndex = 14;
			this.label3.Text = "Blue:";
			this.label3.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// label2
			// 
			this.label2.Location = new System.Drawing.Point(92, 232);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(48, 23);
			this.label2.TabIndex = 12;
			this.label2.Text = "Green:";
			this.label2.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// labelRed
			// 
			this.labelRed.Location = new System.Drawing.Point(4, 232);
			this.labelRed.Name = "labelRed";
			this.labelRed.Size = new System.Drawing.Size(36, 20);
			this.labelRed.TabIndex = 10;
			this.labelRed.Text = "Red:";
			this.labelRed.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// pictColorSample
			// 
			this.pictColorSample.Location = new System.Drawing.Point(80, 276);
			this.pictColorSample.Name = "pictColorSample";
			this.pictColorSample.Size = new System.Drawing.Size(156, 24);
			this.pictColorSample.TabIndex = 7;
			this.pictColorSample.TabStop = false;
			this.toolTipSwatches.SetToolTip(this.pictColorSample, "Custom");
			this.pictColorSample.Paint += new System.Windows.Forms.PaintEventHandler(this.pictColorSamplePaint);
			// 
			// groupColors
			// 
			this.groupColors.Controls.Add(this.panelSwatches);
			this.groupColors.Location = new System.Drawing.Point(4, 4);
			this.groupColors.Name = "groupColors";
			this.groupColors.Size = new System.Drawing.Size(234, 188);
			this.groupColors.TabIndex = 0;
			this.groupColors.TabStop = false;
			this.groupColors.Text = "Basic Colors";
			// 
			// panelSwatches
			// 
			this.panelSwatches.Controls.Add(this.pictColorSwatches);
			this.panelSwatches.Dock = System.Windows.Forms.DockStyle.Fill;
			this.panelSwatches.Location = new System.Drawing.Point(3, 18);
			this.panelSwatches.Name = "panelSwatches";
			this.panelSwatches.Size = new System.Drawing.Size(228, 167);
			this.panelSwatches.TabIndex = 0;
			// 
			// pictColorSwatches
			// 
			this.pictColorSwatches.CaptureArrowKeys = true;
			this.pictColorSwatches.Dock = System.Windows.Forms.DockStyle.Fill;
			this.pictColorSwatches.Image = null;
			this.pictColorSwatches.Location = new System.Drawing.Point(0, 0);
			this.pictColorSwatches.Name = "pictColorSwatches";
			this.pictColorSwatches.Size = new System.Drawing.Size(228, 167);
			this.pictColorSwatches.TabIndex = 0;
			this.toolTipSwatches.SetToolTip(this.pictColorSwatches, "abc");
			this.pictColorSwatches.Enter += new System.EventHandler(this.pictColorSwatchesEnter);
			this.pictColorSwatches.MouseUp += new System.Windows.Forms.MouseEventHandler(this.pictColorSwatchesMouseUp);
			this.pictColorSwatches.KeyDown += new System.Windows.Forms.KeyEventHandler(this.pictColorSwatchesKeyDown);
			this.pictColorSwatches.MouseMove += new System.Windows.Forms.MouseEventHandler(this.pictColorSwatchesMouseMove);
			this.pictColorSwatches.MouseLeave += new System.EventHandler(this.pictColorSwatchesMouseLeave);
			this.pictColorSwatches.Leave += new System.EventHandler(this.pictColorSwathcesLeave);
			// 
			// radioBtnBasic
			// 
			this.radioBtnBasic.Checked = true;
			this.radioBtnBasic.Location = new System.Drawing.Point(64, 0);
			this.radioBtnBasic.Name = "radioBtnBasic";
			this.radioBtnBasic.Size = new System.Drawing.Size(60, 24);
			this.radioBtnBasic.TabIndex = 1;
			this.radioBtnBasic.TabStop = true;
			this.radioBtnBasic.Text = "Basic";
			this.radioBtnBasic.CheckedChanged += new System.EventHandler(this.radioBtnBasicChange);
			// 
			// radioButton2
			// 
			this.radioButton2.Location = new System.Drawing.Point(132, 0);
			this.radioButton2.Name = "radioButton2";
			this.radioButton2.Size = new System.Drawing.Size(82, 24);
			this.radioButton2.TabIndex = 2;
			this.radioButton2.Text = "Web-safe";
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(0, 0);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(52, 23);
			this.label1.TabIndex = 0;
			this.label1.Text = "Colors:";
			this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// label4
			// 
			this.label4.Font = new System.Drawing.Font("Microsoft Sans Serif", 7.8F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.label4.Location = new System.Drawing.Point(8, 276);
			this.label4.Name = "label4";
			this.label4.Size = new System.Drawing.Size(60, 23);
			this.label4.TabIndex = 3;
			this.label4.Text = "COLOR:";
			this.label4.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// panelRadioButtons
			// 
			this.panelRadioButtons.Controls.Add(this.radioBtnBasic);
			this.panelRadioButtons.Controls.Add(this.radioButton2);
			this.panelRadioButtons.Controls.Add(this.label1);
			this.panelRadioButtons.Location = new System.Drawing.Point(0, 192);
			this.panelRadioButtons.Name = "panelRadioButtons";
			this.panelRadioButtons.Size = new System.Drawing.Size(236, 24);
			this.panelRadioButtons.TabIndex = 13;
			// 
			// toolTipSwatches
			// 
			this.toolTipSwatches.ShowAlways = true;
			// 
			// leerBtnConvertColor
			// 
			this.leerBtnConvertColor.AccessibleRole = System.Windows.Forms.AccessibleRole.PushButton;
			this.leerBtnConvertColor.BorderColor = System.Drawing.Color.Black;
			this.leerBtnConvertColor.DisabledColor = System.Drawing.Color.FromArgb(((System.Byte)(180)), ((System.Byte)(177)), ((System.Byte)(163)));
			this.leerBtnConvertColor.DropArrowImage = ((System.Drawing.Image)(resources.GetObject("leerBtnConvertColor.DropArrowImage")));
			this.leerBtnConvertColor.FlatStyle = LeerSoft.Tools.LeerFlatStyle.ThreeD;
			this.leerBtnConvertColor.Font = new System.Drawing.Font("Microsoft Sans Serif", 7.8F);
			this.leerBtnConvertColor.HighlightColor = System.Drawing.Color.LightGray;
			this.leerBtnConvertColor.ImageAlign = System.Drawing.ContentAlignment.MiddleCenter;
			this.leerBtnConvertColor.Location = new System.Drawing.Point(8, 246);
			this.leerBtnConvertColor.Name = "leerBtnConvertColor";
			this.leerBtnConvertColor.Size = new System.Drawing.Size(228, 24);
			this.leerBtnConvertColor.Style = LeerSoft.Tools.LeerButtonStyle.PushButton;
			this.leerBtnConvertColor.TabIndex = 2;
			this.leerBtnConvertColor.Text = "Convert to Basic Color";
			this.leerBtnConvertColor.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			this.leerBtnConvertColor.TextPadding.Bottom = 0;
			this.leerBtnConvertColor.TextPadding.Left = 0;
			this.leerBtnConvertColor.TextPadding.Right = 0;
			this.leerBtnConvertColor.TextPadding.Top = 0;
			this.leerBtnConvertColor.Click += new System.EventHandler(this.leerBtnConvertColorClick);
			// 
			// btnOK
			// 
			this.btnOK.AccessibleRole = System.Windows.Forms.AccessibleRole.PushButton;
			this.btnOK.BorderColor = System.Drawing.Color.Black;
			this.btnOK.DialogResult = System.Windows.Forms.DialogResult.OK;
			this.btnOK.DisabledColor = System.Drawing.Color.FromArgb(((System.Byte)(180)), ((System.Byte)(177)), ((System.Byte)(163)));
			this.btnOK.DropArrowImage = ((System.Drawing.Image)(resources.GetObject("btnOK.DropArrowImage")));
			this.btnOK.FlatStyle = LeerSoft.Tools.LeerFlatStyle.ThreeD;
			this.btnOK.Font = new System.Drawing.Font("Microsoft Sans Serif", 7.8F);
			this.btnOK.HighlightColor = System.Drawing.Color.LightGray;
			this.btnOK.ImageAlign = System.Drawing.ContentAlignment.MiddleCenter;
			this.btnOK.Location = new System.Drawing.Point(20, 308);
			this.btnOK.Name = "btnOK";
			this.btnOK.Size = new System.Drawing.Size(92, 28);
			this.btnOK.Style = LeerSoft.Tools.LeerButtonStyle.PushButton;
			this.btnOK.TabIndex = 4;
			this.btnOK.Text = "OK";
			this.btnOK.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			this.btnOK.TextPadding.Bottom = 0;
			this.btnOK.TextPadding.Left = 0;
			this.btnOK.TextPadding.Right = 0;
			this.btnOK.TextPadding.Top = 0;
			// 
			// btnCancel
			// 
			this.btnCancel.AccessibleRole = System.Windows.Forms.AccessibleRole.PushButton;
			this.btnCancel.BorderColor = System.Drawing.Color.Black;
			this.btnCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.btnCancel.DisabledColor = System.Drawing.Color.FromArgb(((System.Byte)(180)), ((System.Byte)(177)), ((System.Byte)(163)));
			this.btnCancel.DropArrowImage = ((System.Drawing.Image)(resources.GetObject("btnCancel.DropArrowImage")));
			this.btnCancel.FlatStyle = LeerSoft.Tools.LeerFlatStyle.ThreeD;
			this.btnCancel.Font = new System.Drawing.Font("Microsoft Sans Serif", 7.8F);
			this.btnCancel.HighlightColor = System.Drawing.Color.LightGray;
			this.btnCancel.ImageAlign = System.Drawing.ContentAlignment.MiddleCenter;
			this.btnCancel.Location = new System.Drawing.Point(132, 308);
			this.btnCancel.Name = "btnCancel";
			this.btnCancel.Size = new System.Drawing.Size(92, 28);
			this.btnCancel.Style = LeerSoft.Tools.LeerButtonStyle.PushButton;
			this.btnCancel.TabIndex = 5;
			this.btnCancel.Text = "Cancel";
			this.btnCancel.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			this.btnCancel.TextPadding.Bottom = 0;
			this.btnCancel.TextPadding.Left = 0;
			this.btnCancel.TextPadding.Right = 0;
			this.btnCancel.TextPadding.Top = 0;
			// 
			// cbxSortOrder
			// 
			this.cbxSortOrder.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.cbxSortOrder.Items.AddRange(new object[] {
																			 "Sort by color brightness",
																			 "Sort by color hue",
																			 "Sort by color name",
																			 "Sort by color saturation"});
			this.cbxSortOrder.Location = new System.Drawing.Point(8, 218);
			this.cbxSortOrder.Name = "cbxSortOrder";
			this.cbxSortOrder.Size = new System.Drawing.Size(228, 24);
			this.cbxSortOrder.TabIndex = 1;
			this.cbxSortOrder.SelectedIndexChanged += new System.EventHandler(this.cbxSortOrderIndexChanged);
			// 
			// LeerColorPicker
			// 
			this.AutoScale = false;
			this.AutoScaleBaseSize = new System.Drawing.Size(6, 15);
			this.ClientSize = new System.Drawing.Size(528, 338);
			this.Controls.Add(this.cbxSortOrder);
			this.Controls.Add(this.btnCancel);
			this.Controls.Add(this.btnOK);
			this.Controls.Add(this.leerBtnConvertColor);
			this.Controls.Add(this.panelRadioButtons);
			this.Controls.Add(this.label4);
			this.Controls.Add(this.groupColors);
			this.Controls.Add(this.panelCustom);
			this.Controls.Add(this.pictColorSample);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.Fixed3D;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "LeerColorPicker";
			this.ShowInTaskbar = false;
			this.Text = "LeerColorPicker";
			this.Load += new System.EventHandler(this.formLoaded);
			this.panelCustom.ResumeLayout(false);
			this.groupCustom.ResumeLayout(false);
			this.panelCustomSwatches.ResumeLayout(false);
			this.groupColors.ResumeLayout(false);
			this.panelSwatches.ResumeLayout(false);
			this.panelRadioButtons.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		
		/// <summary>
		/// Clears any selected well. Called anytime custom area controls clicked.
		/// </summary>
		private void ClearSelectedWell()
		{
			if(wellSelected >= 0)
			{
				this.DrawWellSelected(wellSelected, false);	// undraw select box
				wellSelected = -1;
			}
			if(customWellSelected >= 0)
			{
				this.DrawCustomWellSelected(customWellSelected, false);	// undraw select box
				customWellSelected = -1;
			}
			UpdateUI();			// update state of add/remove buttons
		}
		
		/// <summary>
		/// Find index of custom color swatch at x/y in pictCustomSwatch
		/// </summary>
		/// <param name="x">x coord</param>
		/// <param name="y">y coord</param>
		/// <returns>index w/i a color array for color at point or -1 if out of range</returns>
		private int GetCustomWellIndexAtPoint(int x, int y)
		{
			int widthUsed = 200;								// width used per well row
			int heightUsed = 40;								// height used per col
			if((x >= widthUsed) || (y >= heightUsed))	// out of range?
				return(-1);
			return((y / 20) * 8 + (x / 25));				// return result
		}
		
		/// <summary>
		/// Find index of color swatch at x/y in pictColorSwatch
		/// </summary>
		/// <param name="x">x coord</param>
		/// <param name="y">y coord</param>
		/// <returns>index w/i a color array for color at point or -1 if out of range</returns>
		private int GetWellIndexAtPoint(int x, int y)
		{
			int widthUsed = wellCols * wellWidth;		// width used per well row
			int heightUsed = wellRows * wellHeight;	// height used per col
			if((x >= widthUsed) || (y >= heightUsed))	// out of range?
				return(-1);
			return((y / wellHeight) * wellCols + (x / wellWidth));	// return result
		}
		
		/// <summary>
		/// Draw the image for basic color selection
		/// </summary>
		private void DrawBasicColorsImage()
		{
			int i, j;
			int row, col;
			int x, y;

			// first clear entire image to background color
			
			Color colBack = pictColorSwatches.BackColor;
			for(i = 0; i < imageBasicColors.Size.Width; ++i)
			{
				for(j = 0; j < imageBasicColors.Size.Height; ++j)
					imageBasicColors.SetPixel(i, j, colBack);
			}			
			
			// now draw the image
			
			for(row = 0; row < 5; ++row)			// loop through image rows
			{
				for(col = 0; col < 8; ++col)		// loop through image columns
				{
					Color colWell = arrayBasicColors[row * 8 + col];
					x = col * 27 + 3;					// offset by space surrounding color swatch
					y = row * 30 + 3;						// compute coords of top/left for this color
					for(i = x; i < x + 21; ++i)
					{
						for(j = y; j < y + 21; ++j)
							imageBasicColors.SetPixel(i, j, colWell);
					}
				}
			}
		}

		/// <summary>
		/// Draw the image for custom color selection
		/// </summary>
		private void DrawCustomColorsImage()
		{
			int i, j;
			int row, col;
			int x, y;

			// first clear entire image to background color
			
			for(i = 0; i < imageCustomColors.Size.Width; ++i)
			{
				for(j = 0; j < imageCustomColors.Size.Height; ++j)
					imageCustomColors.SetPixel(i, j, pictCustomSwatches.BackColor);
			}			
			
			// now draw the image, using colors from CustomColors[] array
			
			for(row = 0; row < 2; ++row)			// loop through image rows
			{
				for(col = 0; col < 8; ++col)		// loop through image columns
				{
					x = col * 25 + 2;						// offset by space surrounding color swatch
					y = row * 20 + 2;						// compute coords of top/left for this color
					for(i = x; i < x + 21; ++i)
					{
						for(j = y; j < y + 16; ++j)
							imageCustomColors.SetPixel(i, j, CustomColors[row * 8 + col]);
					}
				}
			}
		}

		/// <summary>
		/// Draw/undraw focus on selected custom well.
		/// </summary>
		/// <param name="idx">index of well w/i the custom set.</param>
		/// <param name="focus">true if focus to be drawn; false to undraw</param>
		private void DrawCustomWellFocus(int idx, bool focus)
		{
			if(idx < 0)				// well defined?
				return;
			int x = (idx % 8) * 25 + 1;			// x pos for border around target well
			int y = (idx / 8) * 20 + 1;			// y pos border around target well
			Rectangle rect = new Rectangle(x, y, 23, 18); // rect around border	
			Bitmap bmp = imageCustomColors;

			if(focus == true)
			{
				// draw focus rectangle
				
				for(int i = rect.Left; i < rect.Right - 1; ++i)	// draw top line
					bmp.SetPixel(i, rect.Top, Color.White);
				for(int i = rect.Top; i < rect.Bottom - 1; ++i)	// draw left side line
					bmp.SetPixel(rect.Left, i, Color.White);
				for(int i = rect.Left; i < rect.Right; ++i)	// draw bot line
					bmp.SetPixel(i, rect.Bottom - 1, Color.Black);
				for(int i = rect.Top; i < rect.Bottom; ++i)	// draw right side line
					bmp.SetPixel(rect.Right - 1, i, Color.Black);
			}
			else
			{
				// undraw focus rectangle
				
				for(int i = rect.Left; i < rect.Right - 1; ++i)	// draw top line
					bmp.SetPixel(i, rect.Top, pictCustomSwatches.BackColor);
				for(int i = rect.Top; i < rect.Bottom - 1; ++i)	// draw left side line
					bmp.SetPixel(rect.Left, i, pictCustomSwatches.BackColor);
				for(int i = rect.Left; i < rect.Right; ++i)	// draw bot line
					bmp.SetPixel(i, rect.Bottom - 1, pictCustomSwatches.BackColor);
				for(int i = rect.Top; i < rect.Bottom; ++i)	// draw right side line
					bmp.SetPixel(rect.Right - 1, i, pictCustomSwatches.BackColor);
			}
			pictCustomSwatches.Invalidate(rect);		// invalidate region to repaint
		}
		
		/// <summary>
		/// Draw/undraw selected border on selected custom well.
		/// </summary>
		/// <param name="idx">index of well w/i custom set used.</param>
		/// <param name="selected">true if selected border to be drawn; false to undraw</param>
		private void DrawCustomWellSelected(int idx, bool selected)
		{
			if(idx < 0)				// well defined?
				return;
			Bitmap bmp = imageCustomColors;				// get cur image used for color swatches
			DrawCustomWellFocus(idx, false);				// clears previous focus rectangle
			int x = (idx % 8) * 25;							// x pos for border around target well
			int y = (idx / 8) * 20;							// y pos border around target well
			Rectangle rect = new Rectangle(x, y, 25, 20); // rect around border	

			if(selected == true)
			{
				// draw selected rectangle
				
				for(int i = rect.Left; i < rect.Right - 1; ++i)	// draw top line
					bmp.SetPixel(i, rect.Top, Color.Black);
				for(int i = rect.Top; i < rect.Bottom - 1; ++i)	// draw left side line
					bmp.SetPixel(rect.Left, i, Color.Black);
				for(int i = rect.Left; i < rect.Right; ++i)	// draw bot line
					bmp.SetPixel(i, rect.Bottom - 1, Color.Gray);
				for(int i = rect.Top; i < rect.Bottom; ++i)	// draw right side line
					bmp.SetPixel(rect.Right - 1, i, Color.Gray);
			}
			else
			{
				// undraw selected rectangle
				
				for(int i = rect.Left; i < rect.Right - 1; ++i)	// draw top line
					bmp.SetPixel(i, rect.Top, pictCustomSwatches.BackColor);
				for(int i = rect.Top; i < rect.Bottom - 1; ++i)	// draw left side line
					bmp.SetPixel(rect.Left, i, pictCustomSwatches.BackColor);
				for(int i = rect.Left; i < rect.Right; ++i)	// draw bot line
					bmp.SetPixel(i, rect.Bottom - 1, pictCustomSwatches.BackColor);
				for(int i = rect.Top; i < rect.Bottom; ++i)	// draw right side line
					bmp.SetPixel(rect.Right - 1, i, pictCustomSwatches.BackColor);
			}
			pictCustomSwatches.Invalidate(rect);		// invalidate region to repaint
		}
		
		/// <summary>
		/// Draw the pointer image focused at vPos.
		/// </summary>
		/// <param name="fBackground">True means paint entire background</param>
		private void DrawPointerImage(bool fBackground)
		{
			int x, y;
			
			// if fBackground, first fill all pixels with background color
			
			Color c = this.BackColor;	// background color for image
			if(fBackground)
			{
				for(x = 0; x < 9; ++x)
				{
					for(y = 0; y < 217; ++y)
						imageShadePtr.SetPixel(x, y, c);
				}
			}
			else
			{
				// if not doing full background, then must erase pixes where pointer was

				y = lastYforPtr;					// last place pointer was at				
				for(int i = 0; i < 17; ++i, ++y)	// need to paint 17 rows
				{
					int j = Math.Abs(8 - i);	// x position to start
					for(x = j; x < 9; ++x)
						imageShadePtr.SetPixel(x, y, c);
				}
			}
			
			// now draw pointer based on whether focused or not
			
			if(fPointerHasFocus)
				c = Color.Black;			// focused...use black
			else
				c = Color.Silver;
		
			// compute y position as 8  lower than where vPos points
			
			y = 200 - vPos;				// y + 8 is where it points
			lastYforPtr = y;				// save place where pointer being painted for next draw so we can erase
			for(int i = 0; i < 17; ++i, ++y)	// need to paint 17 rows
			{
				int j = Math.Abs(8 - i);	// x position to start
				for(x = j; x < 9; ++x)
					imageShadePtr.SetPixel(x, y, c);
			}
		}

		/// <summary>
		/// Draw image to be used for shades
		/// </summary>
		private void DrawShadesImage()
		{
			int i, y;
			for(i = 200, y = 0; i > 0; --i, ++y)
			{
				Color c = Methods.HsbToColor((xPos / 2) * 3, yPos / 2, i / 2);
				for(int x = 0; x < 16; ++x)
				{
					imageShades.SetPixel(x, y, c);
				}
			}
		}
		
		/// <summary>
		/// Fetches web colors and draws the image for web color selection.
		/// </summary>
		private void DrawWebColorsImage()
		{
			int i, j;
			int row, col;
			int x, y;

			// first clear entire image to background color
			
			Color colBack = pictColorSwatches.BackColor;			
			for(i = 0; i < imageWebColors.Size.Width; ++i)
			{
				for(j = 0; j < imageWebColors.Size.Height; ++j)
					imageWebColors.SetPixel(i, j, colBack);
			}			
			
			// now draw web color image
			
			for(row = 0; row < 10; ++row)			// loop through image rows
			{
				for(col = 0; col < 14; ++col)		// loop through image columns
				{
					Color colWell = arrayWebColors[row * 14 + col];
					x = col * 16 + 2;					// add 1 for space around color swatch
					y = row * 16 + 2;					// compute coords of top/left for this color
					for(i = x; i < x + 13; ++i)
					{
						for(j = y; j < y + 13; ++j)
							imageWebColors.SetPixel(i, j, colWell);
					}
				}
			}
		}

		/// <summary>
		/// Draw/undraw focus on selected well.
		/// </summary>
		/// <param name="idx">index of well w/i the swatch set used.</param>
		/// <param name="focus">true if focus to be drawn; false to undraw</param>
		private void DrawWellFocus(int idx, bool focus)
		{
			if(idx < 0)				// well defined?
				return;
			Bitmap bmp = (Bitmap)pictColorSwatches.Image;		// get cur image used for color swatches
			int x = (idx % wellCols) * wellWidth + wellSpace - 1;	// x pos for border around target well
			int y = (idx / wellCols) * wellHeight + wellSpace - 1;	// y pos border around target well
			Rectangle rect = new Rectangle(x, y, wellSwatchWidth + 2, wellSwatchWidth + 2); // rect around border	

			if(focus == true)
			{
				// draw focus rectangle
				
				for(int i = rect.Left; i < rect.Right - 1; ++i)	// draw top line
					bmp.SetPixel(i, rect.Top, Color.White);
				for(int i = rect.Top; i < rect.Bottom - 1; ++i)	// draw left side line
					bmp.SetPixel(rect.Left, i, Color.White);
				for(int i = rect.Left; i < rect.Right; ++i)	// draw bot line
					bmp.SetPixel(i, rect.Bottom - 1, Color.Black);
				for(int i = rect.Top; i < rect.Bottom; ++i)	// draw right side line
					bmp.SetPixel(rect.Right - 1, i, Color.Black);
			}
			else
			{
				// undraw focus rectangle
				
				for(int i = rect.Left; i < rect.Right - 1; ++i)	// draw top line
					bmp.SetPixel(i, rect.Top, pictColorSwatches.BackColor);
				for(int i = rect.Top; i < rect.Bottom - 1; ++i)	// draw left side line
					bmp.SetPixel(rect.Left, i, pictColorSwatches.BackColor);
				for(int i = rect.Left; i < rect.Right; ++i)	// draw bot line
					bmp.SetPixel(i, rect.Bottom - 1, pictColorSwatches.BackColor);
				for(int i = rect.Top; i < rect.Bottom; ++i)	// draw right side line
					bmp.SetPixel(rect.Right - 1, i, pictColorSwatches.BackColor);
			}
			pictColorSwatches.Invalidate(rect);		// invalidate region to repaint
		}
		
		/// <summary>
		/// Draw/undraw selected border on selected well.
		/// </summary>
		/// <param name="idx">index of well w/i the swatch set used.</param>
		/// <param name="selected">true if selected border to be drawn; false to undraw</param>
		private void DrawWellSelected(int idx, bool selected)
		{
			if(idx < 0)				// well defined?
				return;
			Bitmap bmp = (Bitmap)pictColorSwatches.Image;		// get cur image used for color swatches
			DrawWellFocus(idx, false);									// clears previous focus rectangle
			int x = (idx % wellCols) * wellWidth + wellSpace - 2;	// x pos for border around target well
			int y = (idx / wellCols) * wellHeight + wellSpace - 2;	// y pos border around target well
			Rectangle rect = new Rectangle(x, y, wellSwatchWidth + 4, wellSwatchWidth + 4); // rect around border	

			if(selected == true)
			{
				// draw selected rectangle
				
				for(int i = rect.Left; i < rect.Right - 1; ++i)	// draw top line
					bmp.SetPixel(i, rect.Top, Color.Black);
				for(int i = rect.Top; i < rect.Bottom - 1; ++i)	// draw left side line
					bmp.SetPixel(rect.Left, i, Color.Black);
				for(int i = rect.Left; i < rect.Right; ++i)	// draw bot line
					bmp.SetPixel(i, rect.Bottom - 1, Color.Gray);
				for(int i = rect.Top; i < rect.Bottom; ++i)	// draw right side line
				bmp.SetPixel(rect.Right - 1, i, Color.Gray);
			}
			else
			{
				// undraw selected rectangle
				
				for(int i = rect.Left; i < rect.Right - 1; ++i)	// draw top line
					bmp.SetPixel(i, rect.Top, pictColorSwatches.BackColor);
				for(int i = rect.Top; i < rect.Bottom - 1; ++i)	// draw left side line
					bmp.SetPixel(rect.Left, i, pictColorSwatches.BackColor);
				for(int i = rect.Left; i < rect.Right; ++i)	// draw bot line
					bmp.SetPixel(i, rect.Bottom - 1, pictColorSwatches.BackColor);
				for(int i = rect.Top; i < rect.Bottom; ++i)	// draw right side line
					bmp.SetPixel(rect.Right - 1, i, pictColorSwatches.BackColor);
			}
			pictColorSwatches.Invalidate(rect);		// invalidate region to repaint
		}
		
		/// <summary>
		/// Draw the image for Hue (X axis) and Saturation (Y axis);
		/// </summary>
		private void DrawHueSaturationImage()
		{
			int i, j;
			float iF, jF;
			
			for(i = 1; i <= 240; ++i)			// hue 0-240 (xlated to 0 to 358)
			{
				for(j = 1; j <= 200; ++j)
				{
					iF = i;
					jF = j;		
					// create color at full brightness; convert hue to 0-359 and sat to 0-99
					imageColors.SetPixel(i - 1, j - 1, Methods.HsbToColor((iF / 2) * 3, (jF / 2), 100));
				}
			}
		}
		
		/// <summary>
		/// Cause refresh of the custom color box and shade bar
		/// </summary>
		private void RefreshAll()
		{
			DrawShadesImage();
			DrawPointerImage(false);			// draw only arrow in shade pointer
			pictColors.Refresh();				// refresh color crosshairs
			pictShades.Refresh();				// refresh shade bar
			pictColorSample.Refresh();			// refresh sample color
			pictPointer.Refresh();				// refresh shade pointer
			pictCustomSwatches.Refresh();		// refresh custom colors
			pictColorSwatches.Refresh();		// redisplay image in swatch area
			TryToSelectWell();					// try to select a well matching colorSelected
		}
		
		/// <summary>
		/// Set BG color for buttons
		/// </summary>
		private void SetButtonColor()
		{
			this.btnAddCustom.BackColor = colorButton;
			this.btnCancel.BackColor = colorButton;
			this.btnOK.BackColor = colorButton;
			this.btnRemoveCustom.BackColor = colorButton;
			this.leerBtnConvertColor.BackColor = colorButton;
		}

		/// <summary>
		/// Sort the currently selected color set and recreate the color well image.
		/// </summary>
		private void SortColors()
		{
			if(prevColorSortOrder.Equals(colorSortOrder))	// same sort order last time thru here?
				return;						// get out so we don't waste time redrawing images (thousands of SetPixels!!!)

			// Before sorting, make sure any selected well is deselected
			
			if(wellSelected >= 0)
			{
				DrawWellSelected(wellSelected, false);	// deselect well
				wellSelected = -1;
			}
			
			// Now do the sort
			// First get the comparer for current sort order
			
			IComparer iComp = null;
			switch(colorSortOrder)
			{
				case ColorSortOrder.Brightness:
					iComp = new BrightnessComparer();
					break;
				case ColorSortOrder.Hue:
					iComp = new HueComparer();
					break;
				case ColorSortOrder.Name:
					iComp = new NameComparer();
					break;
				case ColorSortOrder.Saturation:
					iComp = new SaturationComparer();
					break;
			}
			Array.Sort(arrayBasicColors, iComp);	// sort the basic colors
			Array.Sort(arrayWebColors, iComp);		// sort web colors
			DrawBasicColorsImage();						// redraw color images
			DrawWebColorsImage();
			prevColorSortOrder = colorSortOrder;	// save sort order used
		}
		
		/// <summary>
		/// Updates rValue, gValue and bValue from passed color
		/// </summary>
		/// <param name="c"></param>
		private void UpdateRgbFromColor(Color c)
		{
			rValue = c.R;
			gValue = c.G;
			bValue = c.B;
			this.textBoxRed.Text = rValue.ToString();
			this.textBoxGreen.Text = gValue.ToString();
			this.textBoxBlue.Text = bValue.ToString();
		}
		
		/// <summary>
		/// Update RGB text boxes from curent position
		/// </summary>
		private void UpdateRgbTextFromPos()
		{
			Methods.HsbToRgb((xPos / 2) * 3, yPos / 2, vPos / 2, out rValue, out gValue, out bValue);
			colorSelected = Color.FromArgb(rValue, gValue, bValue);	// save selected color
			this.textBoxRed.Text = rValue.ToString();
			this.textBoxGreen.Text = gValue.ToString();
			this.textBoxBlue.Text = bValue.ToString();
		}
		
		/// <summary>
		/// Attempt to select a well if some current swatch matches colorSelected
		/// </summary>
		private void TryToSelectWell()
		{
			if(curSwatchArray == null)			// no swatch array yet?
				return;
			int idx = Array.IndexOf(curSwatchArray, colorSelected); // get idx of colorSelected in curSwatchArray
			if(idx != -1)							// found?
			{
				DrawWellSelected(idx, true);	// select this well here
				wellSelected = idx;				// select this well
			}
		}
		
		/// <summary>
		/// Updates the HSB values from current RGB values (rValue, gValue, bValue)
		/// </summary>
		private void UpdatePosFromRgb()
		{
			float h, s, l;
			Methods.RgbToHsb(rValue, gValue, bValue, out h, out s, out l);
			
			// now convert each parameter to Posistions
			
			xPos = System.Convert.ToInt32((h * 239.0) + 1.0);	// hue to 1-240
			yPos = System.Convert.ToInt32((s * 199.0) + 1.0);	// sat to 1-200
			vPos = System.Convert.ToInt32((l * 199.0) + 1.0);	// bright to 1-200
		}
		
		/// <summary>
		/// Updates UI buttons enable state.
		/// </summary>
		private void UpdateUI()
		{
			// Add custom color button: only if some white available.
			
			bool f = false;
			if(customWellSelected == -1)		// to enable, can't already have one selected
			{
				for(int i = 0; i < 16; ++i)
				{
					if(CustomColors[i].ToArgb() == Color.White.ToArgb())	// available spot found?
					{
						f = true;
						break;
					}
				}
			}
			this.btnAddCustom.Enabled = f;	// enable iff some white
			
			// remove custom color button: one must be selected
			
			this.btnRemoveCustom.Enabled = (customWellSelected >= 0) && (CustomColors[customWellSelected].ToArgb() != Color.White.ToArgb());
		}
		
		/// <summary>
		/// User wants to add current color to custom colors.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void btnAddCustomClick(object sender, System.EventArgs e)
		{
			int i = 0;
			
			// first look to see if color already there. Warn user if so.

			for(i = 0; i < 16; ++i)
			{
				if(CustomColors[i].ToArgb() == colorSelected.ToArgb())	// color there?
				{
					if(MessageBox.Show(ERR_CUSTOMCOLOREXISTS, TIT_CUSTOMCOLOREXISTS, MessageBoxButtons.YesNo,
							MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.No)
					{
						 return;
					}
					break;
				}
			}			
			
			// next look for 1st white custom color array entry
			
			for(i = 0; i < 16; ++i)
			{
				if(CustomColors[i].ToArgb() == Color.White.ToArgb())		// found a white one?
				{
					// found "empty" custom will, fill it up
					
					CustomColors[i] = colorSelected;			// put new color in image
					DrawCustomColorsImage();							// draw new custom image
					pictCustomSwatches.Refresh();						// refresh the color swatches
					UpdateUI();												// fix button states
					return;
				}
			}
			
			// No more custom colors available. Must tell user.
			
			MessageBox.Show(ERR_NOCUSTOMSLOTSOPEN, TIT_NOCUSTOMSLOTSOPEN, MessageBoxButtons.OK, MessageBoxIcon.Error);
		}

		/// <summary>
		/// Remove custom color pressed.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void btnRemoveCustomClick(object sender, System.EventArgs e)
		{
			if(customWellSelected == -1)				// none selected?
				return;
			CustomColors[customWellSelected] = Color.White; // change selected color to white
			DrawCustomColorsImage();					// draw new custom image
			pictCustomSwatches.Refresh();				// and display new colors (this "undraws" select box)
			customWellSelected = -1;					// make sure no custom well thought to be selected
			UpdateUI();										// fix button states
		}

		/// <summary>
		/// User changed selection for color sorting.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void cbxSortOrderIndexChanged(object sender, System.EventArgs e)
		{
			colorSortOrder = (ColorSortOrder)cbxSortOrder.SelectedIndex;	// save new sort order
			SortColors();						// and resort
			pictColorSwatches.Refresh();	// redisplay image in swatch area
			TryToSelectWell();				// try to select a well matching colorSelected
		}


		/// <summary>
		/// Form loaded....draw our main image
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void formLoaded(object sender, System.EventArgs e)
		{
			Methods.PropogateSkin(this, skin);		// propogate this skin
			if(skin == null)								// if no skin given,
				SetButtonColor();							// set colors for buttons
		
			// make sure CustomColors is array of 16 colors
			
			if(CustomColors.Length > 16)				// too many colors?
			{
				Color[] temp = new Color[16];			// temp color array
				Array.Copy(CustomColors, temp, 16);	// copy 1st 16
				CustomColors = new Color[16];			// redo size of custom colors
				Array.Copy(temp, CustomColors, 16);		// redo custom colors
			}
			else if(CustomColors.Length < 16)
			{
				Color[] temp = new Color[16];			// temp color array
				Array.Copy(CustomColors, temp, CustomColors.Length); // copy what we have
				int oldLength = CustomColors.Length;
				CustomColors = new Color[16];
				Array.Copy(temp, CustomColors, 16);	// copy into new customcolor array
				for(int i = oldLength; i < 16; ++i)
					CustomColors[i] = Color.White;		// white for unused colors
			}
			
			// convert initial colorSelected to a name color structure if possible.
			// Must search known colors and compare RGB. Will not include system colors.
			
			if(colorSelected.IsNamedColor)			// only do testing if not already a named color
				this.toolTipSwatches.SetToolTip(this.pictColorSample, colorSelected.Name);
			else
			{
				Array knownColors = Enum.GetValues( typeof(System.Drawing.KnownColor) );
				foreach(KnownColor k in knownColors )
				{
					Color c = Color.FromKnownColor(k);	// get a color
					if(!c.IsSystemColor && (c.A > 0))	// not system or transparent
					{
						if((colorSelected.R==c.R) && (colorSelected.G==c.G) && (colorSelected.B==c.B)) // match RGB?
						{
							colorSelected = c;		// choose this named color instance
							this.toolTipSwatches.SetToolTip(this.pictColorSample, c.Name); // set tool tip
							break;
						}
					}
				}
			}
			this.radioBtnBasic.Checked = StartWithBasic;		// make radio button agree w/property		
			UpdateRgbFromColor(colorSelected);		// set RGB fom selected color
			UpdatePosFromRgb();							// update POS from selected color
			cbxSortOrder.SelectedIndex = (int)colorSortOrder;	// reflect sort order in list box
			DrawHueSaturationImage();					// create custom colors image grid
			DrawCustomColorsImage();					// create custom colors image
			DrawShadesImage();
			DrawPointerImage(true);						// draw pointer image and full background
			this.pictColors.Image = imageColors;	// set image to be used by picture
			curSwatchArray = fUseBasicColors ? arrayBasicColors : arrayWebColors;	// points to swatches currently being used
			this.pictColorSwatches.Image = fUseBasicColors ? imageBasicColors : imageWebColors;
			this.pictCustomSwatches.Image = imageCustomColors;	// display custom colors
			this.pictShades.Image = imageShades;	// image for shades
			this.pictPointer.Image = imageShadePtr;// set image for shade pointer
			SortColors();									// sort the colors
			RefreshAll();
		}

		/// <summary>
		/// Convert the current selected color to the closes color from curSwatchArray
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void leerBtnConvertColorClick(object sender, System.EventArgs e)
		{
			// Technique will be to loop through all known colors in curSwatchArray until will find one in which
			// the target color is within n of each of the r, g and b color values
			
			for(int i = 2; i < 255; i += 5)
			{
				if(i > 250)		// catch all in case we really get here (should not happen)
					i = 255;
				for(int j = 0; j < curSwatchArray.Length; ++j)	// loop through colors
				{
					if((Math.Abs(curSwatchArray[j].R - rValue) <= i) &&
						(Math.Abs(curSwatchArray[j].G - gValue) <= i) &&
						(Math.Abs(curSwatchArray[j].B - bValue) <= i))
					{
						// got a match w/i "i" for each of R, G and B
						
						UpdateRgbFromColor(curSwatchArray[j]);	// set new RGB values
						UpdatePosFromRgb();						// update custom dialog
						RefreshAll();								// refresh everything
						if((wellSelected >= 0) && (wellSelected != j))	// some other well currently selected?
							DrawWellSelected(wellSelected, false);	// should not happen ....maybe
						DrawWellSelected(j, true);				// select this well
						wellSelected = j;							// set as selected well
						colorSelected = curSwatchArray[j];	// save result selected color
						this.toolTipSwatches.SetToolTip(this.pictColorSample, colorSelected.Name);	// custom color now
						return;
					}
				}
			}
		}

		/// <summary>
		/// Key down in color picker. Maybe be cursor key!
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictColorsKeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
		{
			bool fChangeMade = false;
			if(e.KeyData == Keys.Down)
			{
				if(yPos < 200)										// any pos lower?
				{
					++yPos;											// update y pos
					fChangeMade = true;
				}
			}
			if(e.KeyData == Keys.Up)
			{
				if(yPos > 1)										//	any pos higher?
				{
					--yPos;
					fChangeMade = true;
				}
			}
			if(e.KeyData == Keys.Left)
			{
				if(xPos > 1)										// any pos left?
				{
					--xPos;
					fChangeMade = true;
				}
			}
			if(e.KeyData == Keys.Right)
			{
				if(xPos < 240)										// any pos right?
				{
					++xPos;
					fChangeMade = true;
				}
			}
			
			if(fChangeMade == true)
			{
				ClearSelectedWell();			// clear a selected well
				this.toolTipSwatches.SetToolTip(this.pictColorSample, "Custom");	// custom color now
				UpdateRgbTextFromPos();		// update text boxes
				RefreshAll();				// refresh pictures
				e.Handled = true;			// say key handled				
			}
		}

		/// <summary>
		/// Mouse click in image box. Repaint box as needed
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictColorsMouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			ClearSelectedWell();			// clear a selected well
			this.toolTipSwatches.SetToolTip(this.pictColorSample, "Custom");	// custom color now
			xPos = e.X + 1;			// save new coordinates; range 1-360
			yPos = e.Y + 1;			// range 1-200
			UpdateRgbTextFromPos();		// update text boxes
			RefreshAll();				// refresh pictures
			fMouseDown = true;		// mouse is down!
		}
		
		/// <summary>
		/// Paint the image with crosshairs
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictColorsPaint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			Pen p = null;	
			if(fColorsHasFocus)					// if focused, use black pen
				p = new Pen(Color.Black);
			else
				p = new Pen(Color.Silver);		// else use silver pen
			//e.Graphics.DrawImage(imageColors, pictColors.ClientRectangle);	// draw our image
			
			// Draw vertical crosshair; leave room for circle
			e.Graphics.DrawLine(p, xPos, 0, xPos, yPos - 5);
			e.Graphics.DrawLine(p, xPos, yPos + 5, xPos, 200);
			
			// Draw horizonal crosshair; leave room for circle
			e.Graphics.DrawLine(p, 0, yPos, xPos - 5, yPos);
			e.Graphics.DrawLine(p, xPos + 5, yPos, 240, yPos);
			
			// Draw ellipse at intersection of crosshairs
			e.Graphics.DrawEllipse(p, xPos - 5, yPos - 5, 10, 10);
			p.Dispose();
			
			// Cause paint of sample color
			
			base.OnPaint(e);								// let base do his thing
		}

		/// <summary>
		/// Mouse left custom well area. Unfocus a focused custom well.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictCustomSwatchesMouseLeave(object sender, System.EventArgs e)
		{
			if(customWellFocused >= 0)		// something focused?
			{
				DrawCustomWellFocus(customWellFocused, false);	// undraw focus box
				customWellFocused = -1;
			}
		}

		/// <summary>
		/// Mouse moved in custom swatches. Focus the appropriate custom well.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictCustomSwatchesMouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if((e.X < 0) || (e.X >= pictCustomSwatches.Width) || (e.Y < 0) || (e.Y >= pictCustomSwatches.Height)) // out of range
			{
				if(customWellFocused >= 0)						// any well focused?
				{
					DrawCustomWellFocus(customWellFocused, false);		// undraw focus box
					customWellFocused = -1;
				}
				return;												// ignore if out of range
			}
			int idx = GetCustomWellIndexAtPoint(e.X, e.Y);		// which well is this?
			if(idx < 0)												// no well there?
			{
				if(customWellFocused >= 0)						// any well focused?
				{
					DrawCustomWellFocus(customWellFocused, false);		// undraw focus box
					customWellFocused = -1;
				}
				return;
			}
			if((idx == customWellSelected) || (idx == customWellFocused))	// do nothing if already selected or focused
				return;
			DrawCustomWellFocus(customWellFocused, false);		// undraw focus on currently focused well
			DrawCustomWellFocus(idx, true);							// draw focus on new well
			customWellFocused = idx;									// new well focused
		}

		/// <summary>
		/// User clicked on a custom swatch
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictCustomSwatchesMouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if((e.X < 0) || (e.X >= pictCustomSwatches.Width) || (e.Y < 0) || (e.Y >= pictColorSwatches.Height))
			{
				if(customWellFocused >= 0)						// any well focused?
				{
					DrawCustomWellFocus(customWellFocused, false);	// undraw focus box
					customWellFocused = -1;
				}
				return;												// ignore if out of range
			}
			int idx = GetCustomWellIndexAtPoint(e.X, e.Y);		// which well is this?
			if(idx < 0)												// no well there?
			{
				if(customWellFocused >= 0)					// any well focused?
				{
					DrawCustomWellFocus(customWellFocused, false);	// undraw focus box
					customWellFocused = -1;
				}
				return;
			}
			if(idx == customWellSelected)						// do nothing if already selected
				return;
			if(wellSelected >= 0)								// basic well selected?
			{
				DrawWellSelected(wellSelected, false);		// unselect it
				wellSelected = -1;
			}
			if(customWellSelected >= 0)						// any previous well selected?
				DrawCustomWellSelected(customWellSelected, false);		// undraw previous selected well
			if((customWellFocused >= 0) && (idx != customWellFocused))// any previous well focused?	
				DrawCustomWellFocus(customWellFocused, false);			// undraw focus
			customWellFocused = -1;								// no more focused well
			DrawCustomWellSelected(idx, true);				// draw our well as selected
			customWellSelected = idx;							// save selected well
			UpdateRgbFromColor(CustomColors[idx]);	// update current RGB values
			UpdatePosFromRgb();									// update pos for custom dialog
			colorSelected = CustomColors[idx];		// save result selected color
			RefreshAll();
			UpdateUI();												// fix button states
		}

		/// <summary>
		/// Mouse moved. If still down, repaint crosshairs.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictColorsMouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if(fMouseDown)
			{
				xPos = e.X + 1;			// get new coordinates (add one to change range 1-240)
				yPos = e.Y + 1;			// change range to 1-200

				// Don't let x/y be out of range
			
				if(xPos < 1)
					xPos = 1;
				if(xPos >= 240)
					xPos = 240;
				if(yPos < 1)
					yPos = 1;
				if(yPos >= 200)
					yPos = 200;
				UpdateRgbTextFromPos();		// update rgb text boxes
				RefreshAll();				// and repaint
			}
		}

		/// <summary>
		/// Mouse up
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictColorsMouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			fMouseDown = false;							// no repaint here
		}

		/// <summary>
		/// Paint the color sample
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictColorSamplePaint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			// fill the sample with color constructed from current x/y position and brightness value
			
			e.Graphics.FillRectangle(new SolidBrush(Color.FromArgb(rValue, gValue, bValue)), pictColorSample.ClientRectangle);
			
			// now draw web color in sample box
			
			String strCol = String.Format("#{0:X2}{1:X2}{2:X2}", rValue, gValue, bValue);
			Brush br = vPos > 150 ? Brushes.Black : Brushes.White;	// use brush depending on brightness
			SizeF sizeF = e.Graphics.MeasureString(strCol, this.Font);// how much room for string
			RectangleF rectF = pictColorSample.ClientRectangle;
			e.Graphics.DrawString(strCol, this.Font, br, (rectF.Width - sizeF.Width) / 2, (rectF.Height - sizeF.Height) / 2);
		}

		/// <summary>
		/// Focus in shade bar. Insure arrow changes color and redraw.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictShadesEnter(object sender, System.EventArgs e)
		{
			fPointerHasFocus = true;		// pretend in pointer as well so painted black
			DrawPointerImage(false);		// redraw ptr
			pictPointer.Refresh();			// and redisplay
		}

		/// <summary>
		/// Key hit in shades box. Allow cursor movement
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictShadesKeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
		{
			bool fChangeMade = false;
			if(e.KeyData == Keys.Up)
			{
				if(vPos < 200)										// any pos lower?
				{
					++vPos;											// update y pos
					fChangeMade = true;
				}
			}
			if(e.KeyData == Keys.Down)
			{
				if(vPos > 1)										//	any pos higher?
				{
					--vPos;
					fChangeMade = true;
				}
			}
			
			if(fChangeMade == true)
			{
				ClearSelectedWell();			// clear a selected well
				this.toolTipSwatches.SetToolTip(this.pictColorSample, "Custom");	// custom color now
				UpdateRgbTextFromPos();		// update text boxes
				RefreshAll();				// refresh pictures
				e.Handled = true;			// say key handled				
			}
		}

		/// <summary>
		/// Focus out of shade bar. Clear vars and redraw arrow.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictShadesLeave(object sender, System.EventArgs e)
		{
			fPointerHasFocus = false;		// undo so repaints arrow
			DrawPointerImage(false);		// redraw ptr
			pictPointer.Refresh();			// and redisplay
		}

		/// <summary>
		/// User click in shades box; move shade pointer
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictShadesMouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			ClearSelectedWell();			// clear a selected well
			this.toolTipSwatches.SetToolTip(this.pictColorSample, "Custom");	// custom color now
			vPos = 200 - e.Y;				// position relative to shade bar, 1-200
			UpdateRgbTextFromPos();			// update RGB text boxes
			RefreshAll();					// refresh picts
		}

		/// <summary>
		/// Entered pictPointer. Change color of pointer and redraw.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictPointerEnter(object sender, System.EventArgs e)
		{
			fPointerHasFocus = true;		// pointer has focus now
			DrawPointerImage(false);		// redraw ptr
			pictPointer.Refresh();			// and redisplay
		}

		/// <summary>
		/// pointer focus ended. Change color and redraw.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictPointerLeave(object sender, System.EventArgs e)
		{
			fPointerHasFocus = false;		// no focus any more
			DrawPointerImage(false);		// redraw ptr
			pictPointer.Refresh();			// and redisplay
		}

		/// <summary>
		/// Use clicked in shade bar pointer's panel
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictPointerMouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			ClearSelectedWell();			// clear a selected well
			this.toolTipSwatches.SetToolTip(this.pictColorSample, "Custom");	// custom color now
			if(e.Y >= 208)
				vPos = 1;					// force to "1" if out of bounds to low side
			else if(e.Y < 8)
				vPos = 200;					// force to 200 (max) if out of bounds on top
			else
				vPos = 208 - e.Y;			// position relative to shade bar, 1-200 (which starts 8 pixels down)
			fPtrMouseDown = true;
			UpdateRgbTextFromPos();			// update RGB text boxes
			RefreshAll();					// refresh picts
		}

		/// <summary>
		/// Mouse raised after down in shade ptr
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictPointerMouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			fPtrMouseDown = false;
		}

		/// <summary>
		///  mouse moved in shade ptr panel.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictPointerMouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if(fPtrMouseDown == false)		// mouse not down?
				return;							// get out
			if(e.Y >= 208)
				vPos = 1;					// force to "1" if out of bounds to low side
			else if(e.Y < 8)
				vPos = 200;					// force to 200 (max) if out of bounds on top
			else
				vPos = 208 - e.Y;					// new mouse position
			UpdateRgbTextFromPos();			// update RGB text boxes
			RefreshAll();					// refresh picts
		}

		/// <summary>
		/// Blue text box changed. Validate the data.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void textBoxBlueValidate(object sender, System.ComponentModel.CancelEventArgs e)
		{
			bool fError = false;					// in case error
			int newValue = 0;							// in case we have a problem
			try
			{
				newValue = Int32.Parse(textBoxBlue.Text);		// try to save new value
				if((newValue < 0) || (newValue > 255))			// out of range?
				{
					this.textBoxBlue.Text = bValue.ToString();	// reset prev value
					fError = true;					// error!
				}
			}
			catch
			{
				textBoxBlue.Text = bValue.ToString();		// reset to prev value
				fError = true;						// error!
			}

			if(fError == true)					// error occured?
			{
				MessageBox.Show("You must enter an integer between 0-255, inclusive.", "Input Error", MessageBoxButtons.OK,
					MessageBoxIcon.Error);
				e.Cancel = true;					// fail the validation
				return;
			}

			if(bValue == newValue)				// no change??
				return;								// do nothing (don't want to lose color well selection)
			bValue = newValue;					// save new gValue as typed
			UpdatePosFromRgb();					// get new Position from RGB
			ClearSelectedWell();			// clear a selected well
			this.toolTipSwatches.SetToolTip(this.pictColorSample, "Custom");	// custom color now
			RefreshAll();							// refresh the custom color area & sample
		}

		/// <summary>
		/// Green text box changed. Validate the data.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void textBoxGreenValidate(object sender, System.ComponentModel.CancelEventArgs e)
		{
			bool fError = false;					// in case error
			int newValue = 0;							// in case we have a problem
			try
			{
				newValue = Int32.Parse(textBoxGreen.Text);		// try to save new value
				if((newValue < 0) || (newValue > 255))			// out of range?
				{
					this.textBoxGreen.Text = gValue.ToString();	// reset prev value
					fError = true;					// error!
				}
			}
			catch
			{
				textBoxGreen.Text = gValue.ToString();		// reset to prev value
				fError = true;						// error!
			}

			if(fError == true)					// error occured?
			{
				MessageBox.Show("You must enter an integer between 0-255, inclusive.", "Input Error", MessageBoxButtons.OK,
					MessageBoxIcon.Error);
				e.Cancel = true;					// fail the validation
				return;
			}

			if(gValue == newValue)				// no change??
				return;								// do nothing (don't want to lose color well selection)
			gValue = newValue;					// save new gValue as typed
			UpdatePosFromRgb();					// get new Position from RGB
			ClearSelectedWell();			// clear a selected well
			this.toolTipSwatches.SetToolTip(this.pictColorSample, "Custom");	// custom color now
			RefreshAll();							// refresh the custom color area & sample
		}

		/// <summary>
		/// Red text box changed. Validate the data.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void textBoxRedValidating(object sender, System.ComponentModel.CancelEventArgs e)
		{
			bool fError = false;					// in case error
			int newValue = 0;							// in case we have a problem
			try
			{
				newValue = Int32.Parse(textBoxRed.Text);		// try to save new value
				if((newValue < 0) || (newValue > 255))			// out of range?
				{
					this.textBoxRed.Text = rValue.ToString();	// reset prev value
					fError = true;					// error!
				}
			}
			catch
			{
				textBoxRed.Text = rValue.ToString();		// reset to prev value
				fError = true;						// error!
			}

			if(fError == true)					// error occured?
			{
				MessageBox.Show("You must enter an integer between 0-255, inclusive.", "Input Error", MessageBoxButtons.OK,
						MessageBoxIcon.Error);
				e.Cancel = true;					// fail the validation
				return;
			}

			if(rValue == newValue)				// no change??
				return;								// do nothing (don't want to lose color well selection)
			rValue = newValue;					// save new rValue as typed
			UpdatePosFromRgb();					// get new Position from RGB
			ClearSelectedWell();			// clear a selected well
			this.toolTipSwatches.SetToolTip(this.pictColorSample, "Custom");	// custom color now
			RefreshAll();							// refresh the custom color area & sample
		}

		/// <summary>
		/// Entered color swatch area. Must do something so user know we're there.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictColorSwatchesEnter(object sender, System.EventArgs e)
		{
			panelSwatches.BorderStyle = BorderStyle.FixedSingle;
		}

		/// <summary>
		/// Left color swatch area. Reset border.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictColorSwathcesLeave(object sender, System.EventArgs e)
		{
			panelSwatches.BorderStyle = BorderStyle.None;
		}

		/// <summary>
		/// Mouse left swatches...unfocus anything focused
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictColorSwatchesMouseLeave(object sender, System.EventArgs e)
		{
			if(wellFocused >= 0)		// something focused?
			{
				DrawWellFocus(wellFocused, false);	// undraw focus box
				wellFocused = -1;
			}
			toolTipSwatches.SetToolTip(pictColorSwatches, ""); // kill tooltip
		}
		
		/// <summary>
		/// Mouse moved in color swatch area
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictColorSwatchesMouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if((e.X < 0) || (e.X >= pictColorSwatches.Width) || (e.Y < 0) || (e.Y >= pictColorSwatches.Height)) // out of range
			{
				if(wellFocused >= 0)								// any well focused?
				{
					DrawWellFocus(wellFocused, false);		// undraw focus box
					wellFocused = -1;
				}
				toolTipSwatches.SetToolTip(pictColorSwatches, ""); // kill tooltip
				return;												// ignore if out of range
			}
			int idx = GetWellIndexAtPoint(e.X, e.Y);		// which well is this?
			if(idx < 0)												// no well there?
			{
				if(wellFocused >= 0)								// any well focused?
				{
					DrawWellFocus(wellFocused, false);		// undraw focus box
					wellFocused = -1;
				}
				toolTipSwatches.SetToolTip(pictColorSwatches, ""); // kill tooltip
				return;
			}
			if((idx == wellSelected) || (idx == wellFocused))	// do nothing if already selected or focused
			{
				if(idx == wellSelected)							// must do tooltip if selected (focused already has it)
					toolTipSwatches.SetToolTip(pictColorSwatches, curSwatchArray[idx].Name);
				return;
			}
			DrawWellFocus(wellFocused, false);					// undraw focus on currently focused well
			DrawWellFocus(idx, true);								// draw focus on new well
			wellFocused = idx;									// new well focused
			toolTipSwatches.SetToolTip(pictColorSwatches, curSwatchArray[idx].Name); // tooltip to color name now in case we exit
		}

		/// <summary>
		/// Mouse up over image of swatches. Pick the one selected.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictColorSwatchesMouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if((e.X < 0) || (e.X >= pictColorSwatches.Width) || (e.Y < 0) || (e.Y >= pictColorSwatches.Height))
			{
				if(wellFocused >= 0)								// any well focused?
				{
					DrawWellFocus(wellFocused, false);		// undraw focus box
					wellFocused = -1;
				}
				toolTipSwatches.SetToolTip(pictColorSwatches, ""); // kill tooltip
				return;												// ignore if out of range
			}
			int idx = GetWellIndexAtPoint(e.X, e.Y);		// which well is this?
			if(idx < 0)												// no well there?
			{
				if(wellFocused >= 0)								// any well focused?
				{
					DrawWellFocus(wellFocused, false);		// undraw focus box
					wellFocused = -1;
				}
				toolTipSwatches.SetToolTip(pictColorSwatches, ""); // kill tooltip
				return;
			}
			if(idx == wellSelected)								// do nothing if already selected
				return;

			// unselect and basic/web or custom well currently selected

			if(wellSelected >= 0)								// any previous well selected?
				DrawWellSelected(wellSelected, false);		// undraw previous selected well
			if(customWellSelected >= 0)						// custom well selected?
			{
				DrawCustomWellSelected(customWellSelected, false);		// unselect it
				customWellSelected = -1;
				UpdateUI();											// fix add/remove button states
			}
			if((wellFocused >= 0) && (idx != wellFocused))// any previous well focused?	
				DrawWellFocus(wellFocused, false);			// undraw focus
			wellFocused = -1;										// no more focused well

			// Now complete the selection of new well

			DrawWellSelected(idx, true);						// draw our well as selected
			wellSelected = idx;									// save selected well
			toolTipSwatches.SetToolTip(pictColorSwatches, curSwatchArray[idx].Name); // tooltip to color name
			UpdateRgbFromColor(curSwatchArray[idx]);		// update current RGB values
			UpdatePosFromRgb();									// update pos for custom dialog
			colorSelected = curSwatchArray[idx];			// save result selected color
			this.toolTipSwatches.SetToolTip(this.pictColorSample, colorSelected.Name); // tooltip to describe color
			RefreshAll();
		}

		/// <summary>
		/// Key pressed in color swatches. If cursor key, move selected well if any.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictColorSwatchesKeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
		{
			if(wellSelected < 0)		// no well selected?
				return;					// get out
			
			bool fNewWell = false;								// true if well changed
			int row = wellSelected / wellCols;				// get selected well row
			int col = wellSelected - (row * wellCols);	// get selected well column
			if(e.KeyData == Keys.Down)
			{
				if(row < (wellRows - 1))						// any rows lower?
				{
					DrawWellSelected(wellSelected, false);	// undraw selected well
					wellSelected = ++row * wellCols + col;	// new well index
					DrawWellSelected(wellSelected, true);
					fNewWell = true;
				}
			}
			if(e.KeyData == Keys.Up)
			{
				if(row > 0)											//	any rows higher?
				{
					DrawWellSelected(wellSelected, false);	// undraw selected well
					wellSelected = --row * wellCols + col;	// new well index
					DrawWellSelected(wellSelected, true);
					fNewWell = true;
				}
			}
			if(e.KeyData == Keys.Left)
			{
				if(col > 0)											// any cols left?
				{
					DrawWellSelected(wellSelected, false);	// undraw selected well
					wellSelected = row * wellCols + --col;	// new well index
					DrawWellSelected(wellSelected, true);
					fNewWell = true;
				}
			}
			if(e.KeyData == Keys.Right)
			{
				if(col < (wellCols - 1))						// any rows right?
				{
					DrawWellSelected(wellSelected, false);	// undraw selected well
					wellSelected = row * wellCols + col + 1;	// new well index
					DrawWellSelected(wellSelected, true);
					fNewWell = true;
				}
			}
			
			if(fNewWell == true)
			{
				toolTipSwatches.SetToolTip(pictColorSwatches, curSwatchArray[wellSelected].Name); // tooltip to color name
				UpdateRgbFromColor(curSwatchArray[wellSelected]);		// update current RGB values
				UpdatePosFromRgb();									// update pos for custom dialog
				colorSelected = curSwatchArray[wellSelected];			// save result selected color
				this.toolTipSwatches.SetToolTip(this.pictColorSample, colorSelected.Name); // tooltip to describe color
				RefreshAll();
				e.Handled = true;									// key handled
			}
		}

		/// <summary>
		/// focus change...change crosshair color
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictColorsFocusChange(object sender, System.EventArgs e)
		{
			fColorsHasFocus = true;
			pictColors.Refresh();			// refresh will change crosshair colors
		}

		/// <summary>
		/// Lost focus in pictColors
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictColorsLeave(object sender, System.EventArgs e)
		{
			fColorsHasFocus = false;		// no focus in color panel
			pictColors.Refresh();
		}

		/// <summary>
		/// Left custom swatches. Clear border
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pickCustomSwatchesEnter(object sender, System.EventArgs e)
		{
			this.panelCustomSwatches.BorderStyle = BorderStyle.FixedSingle;
		}

		/// <summary>
		/// Key down in custom swatches. Move if we can.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictCustomSwatchesKeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
		{
			if(customWellSelected < 0)		// no well selected?
				return;							// get out
			
			bool fNewWell = false;								// true if well changed
			int row = customWellSelected / wellCols;				// get selected well row
			int col = customWellSelected - (row * wellCols);	// get selected well column
			if(e.KeyData == Keys.Down)
			{
				if(row < 1)										// any rows lower?
				{
					DrawCustomWellSelected(customWellSelected, false);	// undraw selected well
					customWellSelected = ++row * wellCols + col;	// new well index
					DrawCustomWellSelected(customWellSelected, true);
					fNewWell = true;
				}
			}
			if(e.KeyData == Keys.Up)
			{
				if(row > 0)											//	any rows higher?
				{
					DrawCustomWellSelected(customWellSelected, false);	// undraw selected well
					customWellSelected = --row * wellCols + col;	// new well index
					DrawCustomWellSelected(customWellSelected, true);
					fNewWell = true;
				}
			}
			if(e.KeyData == Keys.Left)
			{
				if(col > 0)											// any cols left?
				{
					DrawCustomWellSelected(customWellSelected, false);	// undraw selected well
					customWellSelected = row * wellCols + --col;	// new well index
					DrawCustomWellSelected(customWellSelected, true);
					fNewWell = true;
				}
			}
			if(e.KeyData == Keys.Right)
			{
				if(col < 7)											// any rows right?
				{
					DrawCustomWellSelected(customWellSelected, false);	// undraw selected well
					customWellSelected = row * wellCols + col + 1;	// new well index
					DrawCustomWellSelected(customWellSelected, true);
					fNewWell = true;
				}
			}
			
			if(fNewWell == true)
			{
				UpdateRgbFromColor(CustomColors[customWellSelected]);		// update current RGB values
				UpdatePosFromRgb();									// update pos for custom dialog
				colorSelected = CustomColors[customWellSelected];			// save result selected color
				this.toolTipSwatches.SetToolTip(this.pictColorSample, "Custom"); // tooltip to describe color
				RefreshAll();
				e.Handled = true;									// key handled
				UpdateUI();
			}
		}

		/// <summary>
		/// Focus in custom swatchs. Highlight border
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pictCustomSwatchesLeave(object sender, System.EventArgs e)
		{
			this.panelCustomSwatches.BorderStyle = BorderStyle.None;
		}

		/// <summary>
		/// A radio button has been changed.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void radioBtnBasicChange(object sender, System.EventArgs e)
		{
			// if something selected, deselect it
			
			if(wellSelected >= 0)			// something selected?
			{
				DrawWellSelected(wellSelected, false); // deslect it!
				wellSelected = -1;			// no more selected well
			}

			if(radioBtnBasic.Checked)
			{
				pictColorSwatches.Image = imageBasicColors;	// use basic color image
				groupColors.Text = "Basic Colors";				// change title
				wellCols = 8;					// number of rows for color wells
				wellHeight = 30;				// well cell height
				wellRows = 5;					// number of rows of wells
				wellSpace = 3;					// space around well
				wellSwatchWidth = 21;		// dim of swatch square
				wellWidth = 27;				// width of well
				curSwatchArray = arrayBasicColors;	// array of colors in use
				this.leerBtnConvertColor.Text = "Convert to Basic Color"; //change text of button
				fUseBasicColors = true;		// set so StartWithBasic reflects change
			}
			else
			{
				pictColorSwatches.Image = imageWebColors;		// use web colors
				groupColors.Text = "Web Colors";					// change title
				wellCols = 14;					// number of rows for color wells
				wellHeight = 16;				// well cell height
				wellRows = 10;					// number of rows of wells
				wellSpace = 2;					// space around well
				wellSwatchWidth = 13;		// dim of swatch square
				wellWidth = 16;				// width of well
				curSwatchArray = arrayWebColors; // new array of colors to use
				this.leerBtnConvertColor.Text = "Convert to Web Color"; //change text of button
				fUseBasicColors = false;	// set so StartWithBasic reflects change
			}
			TryToSelectWell();				// attempt to select well if matches colorSelected			
		}

		#region PROPERTIES
		
		/// <summary>
		/// Determines whether the color picker dialog will include radio buttons allowing the end user
		/// to select "basic" or "web-safe" colors.
		/// </summary>
		[Category("Appearance")]
		[Description("Determines if the user will be allowed to select basic and/or web-safe colors.")]
		[Browsable(true)]
		public bool AllowWebColors
		{
			get
			{
				return fAllowWebColors;
			}
			set
			{
				fAllowWebColors = value;
				if(value == false)
				{
					StartWithBasic = true;
				}
				panelRadioButtons.Visible = fAllowWebColors; 
			}
		}
		
		/// <summary>
		/// Determines whether the color picker dialog will display the combo box for selecting
		/// the sort order used for color wells.
		/// </summary>
		[Category("Appearance")]
		[Description("Determines if a color sort spin box is included in the color picker.")]
		[Browsable(true)]
		public bool AllowColorSort
		{
			get
			{
				return fAllowSort;
			}
			set
			{
				fAllowSort = value;
				this.cbxSortOrder.Visible = value;
			}
		}
		
		/// <summary>
		/// Get/Set current button color for dialog buttons. This is usually
		/// left up to the LeerSkin, but can be set when a skin is not used.
		/// </summary>
		[Category("Appearance")]
		[Description("Background color for buttons.")]
		[Browsable(true)]
		public Color ButtonColor
		{
			get
			{
				return colorButton;
			}
			set
			{
				colorButton = value;
			}
		}
		
		/// <summary>
		/// Get/Set the currently selected color.
		/// </summary>
		[Category("Appearance")]
		[Description("Selected color.")]
		[Browsable(true)]
		public Color Color
		{
			get
			{
				return colorSelected;
			}
			set
			{
				colorSelected = value;
			}
		}
		
		/// <summary>
		/// A ColorSortOrder value for the current sort order of color wells.
		/// </summary>
		[Category("Appearance")]
		[Description("Sort order to be used for color wells.")]
		[Browsable(true)]
		public ColorSortOrder ColorSortOrder
		{
			get
			{
				return colorSortOrder;
			}
			set
			{
				if(colorSortOrder == ColorSortOrder.Unknown)		// if setting to unknown, choose Hue
					colorSortOrder = ColorSortOrder.Hue;
				else
					colorSortOrder = value;
				cbxSortOrder.SelectedIndex = (int)colorSortOrder;	// set in sort order box
			}
		}
		
		/// <summary>
		/// Skin. Used for setting basic look-and-feel
		/// </summary>
		[DefaultValue(null)]
		public LeerSoft.Tools.LeerSkin LeerSkin
		{
			get { return(skin); }
			set
			{
				skin = value;					// save passed skin
				if(skin != null)
				{
					this.BackColor = skin.BackColor;		// use skin's look & feel
					this.Font = skin.TextFont;
					this.ForeColor = skin.TextColor;
					this.CustomColors = skin.CustomColors;	// use skin's custom color array
				}
			}
		}
			
		/// <summary>
		/// Indicates whether the color picker should open with basic color wells as opposed to web-safe color wells visible.
		/// </summary>
		[Category("Appearance")]
		[Description("True if the color picker should open with basic color wells as opposed to web-safe color wells visible.")]
		[Browsable(true)]
		public bool StartWithBasic
		{
			get
			{
				return fUseBasicColors;
			}
			set
			{
				fUseBasicColors = value;
				if((value == true) || (fAllowWebColors == false))
				{
					radioBtnBasic.Checked = true; // check this button
				}
			}
		}
		
		#endregion

		#region COLOR COMPARERS
		
		/// <summary>
		/// Color brightness comparer
		/// </summary>
		private class BrightnessComparer : IComparer
		{
			public int Compare(object a, object b)
			{
				int aH, aS, aL, bH, bS, bL;
				Color aColor = (Color)a;
				Color bColor = (Color)b;
				Methods.RgbToHsbInt(aColor.R, aColor.G, aColor.B, out aH, out aS, out aL);
				Methods.RgbToHsbInt(bColor.R, bColor.G, bColor.B, out bH, out bS, out bL);
				if(aL != bL)							// bright diff?
					return(aL.CompareTo(bL));		//  then compare them
				if(aH != bH)							// hue diff?
					return(aH.CompareTo(bH));		//  then compare them next
				return(aS.CompareTo(bS));			// compare sat
			}
		}
		
		/// <summary>
		/// Color Hue comparer
		/// </summary>
		private class HueComparer : IComparer
		{
			public int Compare(object a, object b)
			{
				int aH, aS, aL, bH, bS, bL;
				Color aColor = (Color)a;
				Color bColor = (Color)b;
				Methods.RgbToHsbInt(aColor.R, aColor.G, aColor.B, out aH, out aS, out aL);
				Methods.RgbToHsbInt(bColor.R, bColor.G, bColor.B, out bH, out bS, out bL);
				if(aH != bH)							// hues diff?
					return(aH.CompareTo(bH));		//  then compare them
				if(aS != bS)							// sat diff?
					return(aS.CompareTo(bS));		//  then compare them next
				return(aL.CompareTo(bL));			// compare brightness
			}
		}

		/// <summary>
		/// Color name comparer
		/// </summary>
		private class NameComparer : IComparer
		{
			public int Compare(object a, object b)
			{
				return(((Color)a).Name.CompareTo(((Color)b).Name));
			}
		}
		
		/// <summary>
		/// Color saturation comparer
		/// </summary>
		private class SaturationComparer : IComparer
		{
			public int Compare(object a, object b)
			{
				int aH, aS, aL, bH, bS, bL;
				Color aColor = (Color)a;
				Color bColor = (Color)b;
				Methods.RgbToHsbInt(aColor.R, aColor.G, aColor.B, out aH, out aS, out aL);
				Methods.RgbToHsbInt(bColor.R, bColor.G, bColor.B, out bH, out bS, out bL);
				if(aS != bS)							// sat diff?
					return(aS.CompareTo(bS));		//  then compare them
				if(aH != bH)							// hue diff?
					return(aH.CompareTo(bH));		//  then compare them next
				return(aL.CompareTo(bL));			// compare bright
			}
		}

		#endregion
	
		/// <summary>
		/// Designer class to provide support for Name property coming out right ("shadowing")
		/// </summary>
		internal class LeerColorDialogDesigner : ComponentDesigner
		{
			/// <summary>
			/// Microsoft code...no idea why it works
			/// </summary>
			private String Name
			{
				get { return Component.Site.Name; }
				set { Component.Site.Name = value; }
			}
		
			/// <summary>
			/// No idea what this is
			/// </summary>
			/// <param name="properties"></param>
			protected override void PreFilterProperties(IDictionary properties) 
			{
				base.PreFilterProperties(properties);
				PropertyDescriptor nameProp = (PropertyDescriptor)properties["Name"];
				nameProp = TypeDescriptor.CreateProperty(typeof(LeerColorDialogDesigner), nameProp, new Attribute[0]);
				properties["Name"] = nameProp;
			}
		}
	}
}
